@@ -19,10 +19,14 @@ import logging
 import os.path
 import typing
 from typing import List, Optional, Union
+# from unicodedata import unidata_version
 from uuid import UUID
 
 import requests
 
+from anaml_client.models.user import User, UserId
+from anaml_client.models.user_group import UserGroup, UserGroupName
+
 # Import the optional libraries during type-checking. This allows us to use them
 # in type annotations without insisting that every user install PySpark whether
 # or not they will use it.
@@ -70,8 +74,8 @@ from .models.feature_template import FeatureTemplate, TemplateId, TemplateName
 from .models.feature_template_creation_request import FeatureTemplateCreationRequest
 from .models.file_format import CSV, FileFormat, Parquet, Orc
 from .models.jobs import RunStatus, FeatureStoreRunId, TableMonitoringRunId
-from .models.merge_request import MergeRequest, MergeRequestCreationRequest, MergeRequestId
-from .models.ref import Ref
+from .models.merge_request import MergeRequest, MergeRequestCreationRequest, MergeRequestId, MergeRequestComment
+from .models.ref import Ref, BranchRef
 from .models.table import Table, TableId, TableName
 from .models.aggregate import AggregateExpression
 from .models.attribute import Attribute
@@ -228,6 +232,22 @@ class Anaml:
         result = self._json_or_handle_errors(r)
         return result
 
+    def get_recently_modified_branches(self) -> List[BranchRef]:
+        """Get recently modified branches.
+
+        Returns:
+            A list of BranchRefs with created and updated timestamps
+            associated with the given commit and recently modified.
+        """
+        # this would be nice
+        # r = self._get("/blame/branch")
+        # result = self._json_or_handle_errors(r)
+        branch_commits = [(b['name'], b['head']['createdAt'][:10]) for b in self.get_branches()]
+        recently_commited_branches = \
+            [b[0] for b in branch_commits if (datetime.now() - datetime.strptime(b[1], "%Y-%m-%d")).days < 30]
+
+        return recently_commited_branches
+
     def get_recently_modified_entities(self) -> List[EntityCreatedUpdated]:
         """Get recently modified entity ids, with created and updated timestamps.
 
@@ -921,6 +941,39 @@ class Anaml:
         result = self._json_or_handle_errors(r)
         return MergeRequest.from_json(result)
 
+    def get_merge_request_comments(self, merge_request_id: Union[MergeRequestId, int]) -> List[MergeRequestComment]:
+        """List merge requests from the Anaml server.
+
+        Args:
+            limit: The maximum number of objects to return.
+
+        Returns:
+            A list of merge requests.
+        """
+        r = self._get(f"/merge-request/{int(merge_request_id)}/comment")
+        result = self._json_or_handle_errors(r)
+        return [
+            MergeRequestComment.from_json(r) for r in result
+        ]
+
+    def create_merge_request_comment(
+            self,
+            merge_request_id: Union[MergeRequestId, int],
+            comment: MergeRequestComment
+    ) -> List[MergeRequestComment]:
+        """Create a new Comment for a merge request.
+
+        Args:
+            merge_request_id: The merge request to comment on.
+            comment: The comment to create.
+
+        Returns:
+            The new list of comments for the merge request.
+        """
+        r = self._post(f"/merge-request/{str(merge_request_id)}/comment", json=comment.to_json())
+        self._int_or_handle_errors(r)
+        return self.get_merge_request_comments(merge_request_id)
+
     # Checks
     def get_checks(self, commit_id: Union[CommitId, UUID]) -> List[Check]:
         """Get checks for a given commit from the Anaml server.
@@ -1111,6 +1164,8 @@ class Anaml:
 
         r = self._get(f"/table-monitoring/table/{int(table_id)}/results")
         results = self._json_or_handle_errors(r)
+        if len(results) == 0:
+            raise ValueError(f"Latest results not found for {table_name}")
 
         latest_result_id = results[0]['id']
         r = self._get(f"/table-monitoring/results/{int(latest_result_id)}")
@@ -1121,6 +1176,52 @@ class Anaml:
         else:
             return MonitoringResultPartial.from_json(latest_result)
 
+    # User / User Group functions
+
+    def get_user_from_id(self, user_id: Union[UserId, int]) -> User:
+        """Gets a User object for a given user ID.
+
+        Args:
+            user_id  : UserId of the user to be returned
+
+        Returns:
+            User object for given id
+        """
+        r = self._get(f"/user/{int(user_id)}")
+        result = self._json_or_handle_errors(r)
+
+        return User.from_json(result)
+
+    def get_user_group_by_name(self, name: Union[UserGroupName, str]) -> UserGroup:
+        """Get a UserGroup from the Anaml server.
+
+        Args:
+            name: Name of the UserGroup to retrieve.
+
+        Returns:
+            The requested UserGroup, if it exists.
+        """
+        r = self._get("/user-group", query={'name': str(name)})
+        result = self._json_or_handle_errors(r)
+        return UserGroup.from_json(result)
+
+    def get_users_for_group(
+        self,
+        user_group_name: Optional[Union[UserGroupName, str]] = None
+    ) -> List(User):
+        """Get a count of human users registered on the anaml deployment.
+
+        Args:
+            user_group_name : the name of the group that serves as the human user parent group.
+
+        Returns:
+            length of members attribute of provided group
+        """
+        members = self.get_user_group_by_name(user_group_name).members
+        users = [self.get_user_from_id(m.userId.value) for m in members]
+
+        return users
+
     # Builder functions
     def build_event_feature(
         self,
